import pandas as pd

from bokeh.plotting import figure
from bokeh.io import save, output_file
from bokeh.models import ColumnDataSource, HoverTool

def adj_boxplot(self,q1,q3,skew, dev, Ks, Ke):
    IQD = q3 - q1
    if IQD == 0:
        IQD = dev
    return q1 - Ke*IQD*np.exp(-Ks*skew1), q3 + Ke*IQD*np.exp(Ks*skew1)

def std_boxplot(self,m, dev, K):
    return m -dev*K, m + dev*K

def iqd_boxplot(self,q1,q3,K = 1.5):
    IQD = q3 - q1
    if IQD == 0:
        IQD = dev
    return q1 - K*IQD, q3 + K*IQD

def hist2array(self,xm, n):
    x = [[ xi]*int(ni) for (xi, ni) in zip(xm, n)]
    return np.hstack(x)

def hist2array_cut(self,xm, n, rcut = 0.001):
    Nt = np.sum(n)
    ncum = np.cumsum(n/Nt)
    i_nr0i = np.argwhere( ncum <= rcut ).flatten()
    i_nr0d = np.argwhere( ncum >= (1-rcut)).flatten()
    i_nr0 = np.concatenate((i_nr0i, i_nr0d))
    i_nr1 = np.argwhere((ncum > rcut) & (ncum < 1-rcut)).flatten()
    n_r0 = n[i_nr0]
    xm_r0 = xm[i_nr0]
    max_r0id = [n[i_nr0i[-1]], n[i_nr0d[0]]]
    if 0 in max_r0id:
        max_r0id.remove(0)
    k_r0 = np.min(max_r0id)
    print(k_r0)
    n_r1 = np.round(n[i_nr1]/k_r0)
    xm_r1 = xm[i_nr1]
    x_k0 = hist2array(xm_r0, n_r0)
    x_k1 = hist2array(xm_r1, n_r1)
    return x_k0, x_k1
        
def interactive_histogram(l, r, y, title, x_axis_label, x_tooltip):

    # Column data source
    arr_df = pd.DataFrame({'count': y, 'left': l, 'right': r})
    arr_df['f_count'] = ['%d' % count for count in y]
    
    # column data source
    arr_src = ColumnDataSource(arr_df)

    # Set up the figure same as before
    p = figure(plot_width = 1100, 
               plot_height = 600,
               title = title,
               x_axis_label = x_axis_label, 
               y_axis_label = 'Count')

    # Add a quad glyph with source this time
    p.quad(bottom=0, 
           top='count', 
           left='left', 
           right='right', 
           source=arr_src,
           fill_color='red',
           hover_fill_alpha=0.7,
           hover_fill_color='blue',
           line_color='black')

    # Add style to the plot
    p.title.align = 'center'
    p.title.text_font_size = '18pt'
    p.xaxis.axis_label_text_font_size = '12pt'
    p.xaxis.major_label_text_font_size = '12pt'
    p.yaxis.axis_label_text_font_size = '12pt'
    p.yaxis.major_label_text_font_size = '12pt'

    # Add a hover tool referring to the formatted columns
    hover = HoverTool(tooltips = [('xlabel', '@right'),
                                  ('Count', '@f_count')])

    # Add the hover tool to the graph
    p.add_tools(hover)

    return p

## MAIN
with open('test_hist.csv', 'r') as fp:

   line = 'a'
      
   while line:
       
       line0 = fp.readline()
       line1 = fp.readline()
       
       ls0 = line0.split(',')
       ls1 = line1.split(',')

       if len(ls0) == len(ls1) and len(ls0) > 2:
         x = [ float(v.rstrip()) for v in ls0[1:len(ls0)] ]
         y = [ float(v.rstrip()) for v in ls1[1:len(ls1)] ]

         xdist = abs(x[0]-x[1])
         right = [ float(v)+xdist for v in ls0[1:len(ls0)] ]

         title=ls0[0].split('_')[0]
         x_axis_label='xlabel'
         x_tooltip='tooltip'

         bp = interactive_histogram(x, right, y, title, x_axis_label, x_tooltip)
         output_file('histogram_{}.html'.format(title))
         save(bp)